@@ -13,15 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function
-
 import subprocess
 import sys
 
 from pkg_resources import safe_name, parse_requirements
 
-PY3 = sys.version_info > (3, 0)
-
 
 clients_requirements_table = {
     'install_requires': [
@@ -29,10 +25,7 @@ clients_requirements_table = {
         'urllib3',
         'dogpile.cache',
         'tabulate',
-        'six',
         'jsonschema',
-        'typing',
-        'enum34',
     ],
     'ssh': ['paramiko'],
     'kerberos': [
@@ -42,7 +35,6 @@ clients_requirements_table = {
     ],
     'swift': ['python-swiftclient'],
     'argcomplete': ['argcomplete'],
-    's3': ['boto'],
     'sftp': ['pysftp'],
     # 'gfal2': ['gfal2-python'],
     'dumper': [
@@ -61,10 +53,7 @@ dev_requirements = [
     'virtualenv',
     'xmltodict',
     'pytz',
-    'subprocess32',
     'pycodestyle',
-    'mock',
-    's3cmd',
     'pydoc-markdown',
     'docspec_python',
     'sh',
@@ -81,12 +70,12 @@ server_requirements_table = {
         'pysftp',
         'sqlalchemy',
         'alembic',
-        'python-memcached',
+        'pymemcache',
         'python-dateutil',
         'stomp.py',
         'statsd',
         'geoip2',
-        'oauth2client',
+        'google-auth',
         'retrying',
         'redis',
         'flask',
@@ -157,20 +146,13 @@ def match_define_requirements(requirements_table):
         for req in parse_requirements(fhandle.readlines()):
             if req.key in req_table_by_key:
                 for group in req_table_by_key[req.key]:
-                    if PY3:
-                        print("requirement found", group, req, file=sys.stderr)
-                    else:
-                        sys.stderr.write("requirement found " + str(group) + " " + str(req) + "\n")
-
+                    print("requirement found", group, req, file=sys.stderr)
                     if group == 'install_requires':
                         install_requires.append(str(req))
                     else:
                         extras_require[group].append(str(req))
             else:
-                if PY3:
-                    print("requirement unused", req, "(from " + req.key + ")", file=sys.stderr)
-                else:
-                    sys.stderr.write("requirement unused " + str(req) + " (from " + str(req.key) + ")\n")
+                print("requirement unused", req, "(from " + req.key + ")", file=sys.stderr)
         sys.stderr.flush()
 
     for extra, deps in extras_require.items():
